---
title: "NuGet (.NET) Packages and Feeds in Proget"
nav-title: "NuGet (.NET)"
order: 3
---

NuGet is a package format developed by Microsoft to distribute free and open source. NET libraries. Usually, these packages are publicly available on NuGet.org and are used by Visual Studio or the nuget.exe command-line client.

- [NuGet command-line Reference](https://docs.microsoft.com/en-us/nuget/tools/nuget-exe-cli-reference)

ProGet implements the [NuGet Server API](https://learn.microsoft.com/en-us/nuget/api/overview), referred to as the JSON-LD (v3) API and represented by the endpoint `«proget-server»/nuget/«feed-name»/v3/index.json`

By creating a [feed](/docs/proget/feeds/feed-overview) in ProGet, you can use NuGet packages to distribute private proprietary libraries within your organization. Then configure [Visual Studio](/docs/proget/feeds/nuget/howto-nuget-proxy#add-visual-studio) or the [NuGet CLI](/docs/proget/feeds/nuget/howto-nuget-proxy#add-cli) to use your feed as a new package source.

You can also [use ProGet as a proxy for NuGet.org](/docs/proget/feeds/nuget/howto-nuget-proxy), both to block [improper licenses](/docs/proget/sca/licenses) and [security vulnerabilities](/docs/proget/sca/vulnerabilities), and to cache packages in case the Internet or NuGet.org goes down.

## Installing Packages

NuGet packages can be installed using either the [NuGet CLI](https://docs.microsoft.com/en-us/nuget/quickstart/install-and-use-a-package-using-the-dotnet-cli) or Visual Studio. To install a package from a ProGet feed in the NuGet CLI, `use the dotnet nuget install` command:

```bash
$ dotnet nuget install «package-name» -Version «package-version» -Source http://«proget-server»/nuget/«feed-name»/
```

For Visual Studio you will first need to make sure your ProGet Feed is [set as a source](/docs/proget/feeds/nuget/howto-nuget-proxy#add-visual-studio). Then Right-click on your project in the Solution Explorer and select Manage NuGet Packages.

In the NuGet Package Manager window go to the "Browse" tab and search for the package you want to install. Then select the package from the list and click "Install".

After installing, it will be added to your project’s dependencies, and you can start using it in your code.

## Authenticating to NuGet Feeds

By default, NuGet feeds in ProGet do not need to be authenticated to when consuming packages. However, you can set a feed to require authentication to access, which you will need to do if you plan to publish packages to the feed.

Feeds can be authenticated with a "psedo key" (`«username»:«password»`) but we strongly recommend using an API key for this, with `api` as the username, and then API Key as the password. You can read more about creating one on our [API Key](/docs/proget/reference-api/proget-apikeys) page. 

You can [Create ProGet API Key](/docs/proget/reference-api/proget-apikeys) with Feed Access; you can further restrict this key by associating it to a user you've already given specific permissions

When creating an API Key, fill in the fields by selecting "Feeds ("Use Certain Feeds)" as the "Feed Type" and selecting the NuGet feed to authenticate to. Make sure that the required permission boxes are checked, and then select "Save".

![](/resources/docs/proget-nuget-api.png){height="" width="50%"}

To add and authenticate the feed in The NuGet CLI, enter the following:

```bash
$ dotnet nuget setApiKey «api-key» -Source «feed-url»
```

### Authenticating in Visual Studio

Teams can require authentication for feeds in ProGet, which means that users must log in to their package source in ProGet. To do this, You will first need to create a "Personal API Key". 

Under the user icon in the top-right of ProGet, select “Personal API Keys” and then click "Create Personal API Key". Next, in the pop-up, type an appropriate display name and click "Create API Key". By clicking on the display name, the pop-up will appear again, this time with the Personal API key generated.

![proget-createpersonalapikey.png](/resources/docs/proget-createpersonalapikey.png)

Then When checking the connection of a private NuGet feed after [adding it as a source](/docs/proget/feeds/nuget/howto-nuget-proxy#add-visual-studio) in Visual Studio, a pop-up window will appear asking you to connect to the server. Enter `api` as the username and the personal API key as the password, and then click "OK".

![visualstudio-connecttoserver-inputted.png](/resources/docs/visualstudio-connecttoserver-inputted.png)

## Creating and Publishing NuGet Packages 

The [NuGet package format](https://docs.microsoft.com/en-us/nuget/schema/nuspec) is well-documented, and you can create packages in a number of ways, including [directly from Visual Studio](https://docs.microsoft.com/en-us/nuget/guides/create-net-standard-packages-vs2017) or using [nuget.exe](https://docs.microsoft.com/en-us/nuget/create-packages/creating-a-package). 

You can read more about creating and publishing packages in our [HOWTO: Publish NuGet Packages to a Private Repository in ProGet](/docs/proget/feeds/nuget/howto-nuget-publish) guide.

## Repackaging NuGet Packages { #repackaging-packages }

[Repackaging](/docs/proget/packages/repackaging) lets you create production-ready packages from tested and verified pre-release versions. These "repackaged" packages have identical content, and ProGet maintains an audit trail within the package to track the original versions, who performed the operation, and when.

ProGet supports NuGet repackaging natively. The easiest way to do this is to use the [`packages repackage`](/docs/proget/reference-api/proget-api-packages/proget-api-packages-repackage) command in [pgutil](/docs/proget/reference-api/proget-pgutil). For example, if repackaging version `1.2.3-beta1` of the package `MyPackage` to version `1.2.3`, in the feed `internal-nuget` you would enter:

```bash
$ pgutil packages repackage --feed=internal-nuget --package=MyPackage --version=1.2.3-beta1 --new-version=1.2.3
```

## Symbols and Source Packages 

A NuGet feed in ProGet may be configured as a Symbol/Source server compatible with debuggers such as Visual Studio and WinDbg. See [Symbols and Source Code in ProGet](/docs/proget/feeds/nuget/symbol-and-source-server) for more information.

## Vulnerability Integration

Beginning in ProGet 2023.5, we have added support to show [vulnerability information](https://devblogs.microsoft.com/nuget/how-to-scan-nuget-packages-for-security-vulnerabilities/) directly in Visual Studio and the NuGet CLI.  You will see the severity of your vulnerability along with a link to see the vulnerability details in ProGet.

When selecting a package version with a vulnerability in Visual Studio, you will see the severity with a clickable link to the vulnerability in ProGet.

![](/resources/docs/proget-nuget-vulnerability-visual-studio.png){height="" width=""}

When running `dotnet list package --vulnerable` in the NuGet CLI, you will see a list of vulnerable packages with their severity and a link to the vulnerability in ProGet.

![](/resources/docs/proget-nuget-vulnerability-nuget-cli.png){height="" width=""}

:::(INFO)
⚠ ProGet Free edition limits vulnerability information shown in Visual Studio and the NuGet CLI.
:::

## Managing Package Deprecation, Deletion, and Unlisting

ProGet will show deprecation information directly in Visual Studio and the NuGet CLI. You will see if a specific version of the package was deprecated along with the specified reason. This works for both packages that come from a NuGet connector and packages that were deprecated using the ProGet UI.

When selecting a package version that has been deprecated in Visual Studio, you will see a warning that the package has been deprecated. If the package has been installed in your project, you will also see a shield in the package list.

![](/resources/docs/nuget-deprecated-visualstudio.png){height="" width=""}

When running `dotnet list package --deprecated` in the NuGet CLI, you will see a list of deprecated packages installed in your project along with the deprecation reason.

![](/resources/docs/nuget-deprecated-cli.png){height="" width=""}

Visual Studio and the NuGet CLI heavily caches this information and may take time to show this information after a package has been marked as deprecated. Due to this caching, parts of Visual Studio (version drop-down, version details, and package list) may show the deprecation information at different times. 

To force the NuGet CLI to update its deprecated packages, you will need to first clear the NuGet HTTP Cache using the command: 

```bash
`dotnet nuget locals http-cache --clear`
```

Then the next time you run `dotnet list package --deprecated,` the deprecation information will show immediately.

To force Visual Studio to update its deprecated packages, you will need to navigate to "Tools" > "Options" > "NuGet Package Manager" > "General" and then click "Clear All NuGet Storage". This will clear the entire NuGet cache (including cached packages) from the machine and then the deprecated information will show immediately.

In addition to marking packages as deprecated, ProGet allows you to unlist or delete packages from your feed:

### Unlisting Packages

Unlisted packages will no longer appear in most search results or Visual Studio. However, these packages can still be installed if the exact version number is specified, making it a useful option for deprecated or outdated packages you want to limit to advanced users.

To unlist a package you can use the [`packages status unlisted`](/docs/proget/reference-api/proget-api-packages/proget-api-packages-status) command in [pgutil](/docs/proget/reference-api/proget-pgutil). For example, if unlisting version `1.2.3` of the package `MyPackage` from the feed `internal-nuget` you would enter:

```bash
$ pgutil packages status unlisted  --feed=internal-nuget --package=MyPackage --version=1.2.3 --state=listed
```

### Deleting Packages
Deleting a package will make it permanently unavailable for installation and reference, impacting all users relying on that package version. Before deletion, make sure no active projects depend on it, as removal could lead to build or runtime failures.

To delete a package you can use the [`packages delete`](/docs/proget/reference-api/proget-api-packages/proget-api-packages-delete) command in [pgutil](/docs/proget/reference-api/proget-pgutil). For example, if deleting version `1.2.3` of the package `MyPackage` from the feed `internal-nuget` you would enter:

```bash
$ pgutil packages delete --feed=internal-nuget --package=MyPackage --version=1.2.3
```

## Troubleshooting

## My CI build can’t find a recently published package
The NuGet client maintains a 30-minute HTTP cache for package restore responses. In situations where you deploy more than one new version of a package within 30 minutes, and produce a CI build referencing each of those packages within that 30-minute window, your build will likely fail with a package not found error. This is a known issue in NuGet, and can be worked around by deleting the http cache on your build server prior to restoring packages:

```bash
$ dotnet nuget locals http-cache --clear
```

See [NuGet GitHub Issue 3116](https://github.com/NuGet/Home/issues/3116) for more information about this error.